#include "stdafx.h"
#include <fstream.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <math.h>

#define TEX_WIDTH	128
#define TEX_HEIGHT	128
#define NUM_TEX		2

UINT timer;
GLuint textures[NUM_TEX];
double angle = 0;
HDC hdc;

void SetDCPixelFormat(HDC hdc)
{
	static PIXELFORMATDESCRIPTOR pfd =
	{
		sizeof(PIXELFORMATDESCRIPTOR),          // sizeof this structure
		1,                                      // version number
		PFD_DRAW_TO_WINDOW | 
		PFD_SUPPORT_OPENGL | 
		PFD_DOUBLEBUFFER | 
		PFD_GENERIC_ACCELERATED | 
		PFD_SWAP_EXCHANGE,                      // flags
		PFD_TYPE_RGBA,                          // RGBA pixel values
		32,                                     // 32-bit color
		0, 0, 0, 0, 0, 0,                       // don't care about these
		0, 0,                                   // no alpha buffer
		0, 0, 0, 0, 0,                          // no accumulation buffer
		32,                                     // 32-bit depth buffer
		0,                                      // no stencil buffer
		0,                                      // no auxiliary buffers
		PFD_MAIN_PLANE,                         // layer type
		0,                                      // reserved(must be 0)
		0, 0, 0                                 // no layer masks
	};
	int pixelFormat;

	pixelFormat = ChoosePixelFormat(hdc, &pfd);
	SetPixelFormat(hdc, pixelFormat, &pfd);
}

void InitGL()
{
	GLfloat lightAmbient[] = {0.1f, 0.1f, 0.1f, 1.0f};
	GLfloat lightDiffuse[] = {0.7f, 0.7f, 0.7f, 1.0f};
	GLfloat lightSpecular[] = {0.3f, 0.3f, 0.3f, 1.0f};

	glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightSpecular);
	glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
	glFrontFace(GL_CW);
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);
	glShadeModel(GL_SMOOTH);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	glClearColor(0.0, 0.0, 0.0, 0.0);
	glEnable(GL_TEXTURE_2D);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
}

void DrawTwoPlanes()
{
	glBegin(GL_QUADS);
		glNormal3f(0.0, 0.0, -1);
		glTexCoord1f(0.0);
		glVertex3f(-1, -1, -1);
		glTexCoord1f(1.0);
		glVertex3f(1, -1, -1);
		glTexCoord2f(1.0, 1.0);
		glVertex3f(1, 1, -1);
		glTexCoord2f(0.0, 1.0);
		glVertex3f(-1, 1, -1);
		glNormal3f(0.0, 0.0, 1);
		glTexCoord1f(0.0);
		glVertex3f(-1, 1, 1);
		glTexCoord1f(1.0);
		glVertex3f(1, 1, 1);
		glTexCoord2f(1.0, 1.0);
		glVertex3f(1, -1, 1);
		glTexCoord2f(0.0, 1.0);
		glVertex3f(-1, -1, 1);
	glEnd();
}

void DrawScene(HDC hDC)
{
	GLfloat white[] = {1.0, 1.0, 1.0, 1.0};

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -8.0);
	glRotatef((GLfloat)angle, 0.0, 1.0, 0.0);
	glRotatef((GLfloat)angle, 1.0, 0.0, 0.0);

	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, white);
	glBindTexture(GL_TEXTURE_2D, textures[0]);
    DrawTwoPlanes();
	glRotatef(90.0, 0.0, 1.0, 0.0);
	glBindTexture(GL_TEXTURE_2D, textures[1]);
	DrawTwoPlanes();
	glRotatef(90.0, 1.0, 0.0, 0.0);
	DrawTwoPlanes();

	glFlush();
	SwapBuffers(hDC);
}

void LoadTextures()
{
	int size = TEX_WIDTH * TEX_HEIGHT, c;
	unsigned char pal[1024];
	unsigned char *data = new unsigned char[size * 3];
	unsigned char *buf = new unsigned char[size];
	char filename[] = "textureX.bmp";

	glGenTextures(NUM_TEX, textures);
	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	for (int i = 0; i < NUM_TEX; i++)
	{
		glBindTexture(GL_TEXTURE_2D, textures[i]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

		filename[7] = i + '1';
		ifstream in(filename, ios::binary);
		in.seekg(0x36, ios::beg);
		in.read(pal, sizeof(pal));
		in.read(buf, size);
		in.close();

		for (int j = size - 1; j; j--)
		{
			c = buf[size - j - 1];
			data[j * 3] = pal[(c << 2) + 2];
			data[j * 3 + 1] = pal[(c << 2) + 1];
			data[j * 3 + 2] = pal[c << 2];
		}

		glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_WIDTH, TEX_HEIGHT, 0, 
			GL_RGB, GL_UNSIGNED_BYTE, data);      
    }
	
	delete buf;
	delete data;
}

LONG WINAPI WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	static HGLRC hrc;
	PAINTSTRUCT ps;
	GLdouble aspect;
	GLsizei width, height;

	switch (msg)
	{
	    case WM_CREATE:
			hdc = GetDC(hWnd);
            SetDCPixelFormat(hdc);
            hrc = wglCreateContext(hdc);
			wglMakeCurrent(hdc, hrc);
			InitGL();
			break;

		case WM_SIZE:
			width = (GLsizei)LOWORD(lParam);
			height = (GLsizei)HIWORD(lParam);
			if (!width || !height) return 0;
			aspect = (GLdouble)width /(GLdouble)height;
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			gluPerspective(30.0, aspect, 1, 16.0);
			glViewport(0, 0, width, height);
			LoadTextures();
			break;

		case WM_PAINT:
			BeginPaint(hWnd, &ps);
			EndPaint(hWnd, &ps);
			break;

		case WM_DESTROY:
			wglMakeCurrent(NULL, NULL);
			wglDeleteContext(hrc);
			ReleaseDC(hWnd, hdc);
			glDeleteTextures(3, textures);
			PostQuitMessage(0);
			break;

		case WM_CHAR:
			if (wParam == 27)
				DestroyWindow(hWnd);
			break;

		case WM_ACTIVATE:
			switch (LOWORD(wParam))
			{
				case WA_ACTIVE:
				case WA_CLICKACTIVE:
				{
					DEVMODE devmode;
					devmode.dmBitsPerPel = 32;
					devmode.dmPelsWidth = 640;
					devmode.dmPelsHeight = 480;
					devmode.dmDisplayFrequency = 100;
					devmode.dmFields = DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT | DM_DISPLAYFREQUENCY;
					ChangeDisplaySettings(&devmode, CDS_FULLSCREEN);
					SetCursor(NULL);
					break;
				}
				case WA_INACTIVE:
					ChangeDisplaySettings(NULL, 0);
			}
			break;

		default:
			return DefWindowProc(hWnd, msg, wParam, lParam);
	}

	return 0;
}

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	static char AppName[] = " ";
	static char ClassName[] = "Cube3D";
	WNDCLASS wc;
	HWND hWnd;
	MSG msg;

	wc.style = 0;
	wc.lpfnWndProc =(WNDPROC)WndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	wc.hCursor = NULL;//LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszMenuName = (LPCTSTR)0;
	wc.lpszClassName = ClassName;

	RegisterClass(&wc);

	hWnd = CreateWindow(ClassName, AppName,
						WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
						CW_USEDEFAULT, CW_USEDEFAULT, 640,
						480, HWND_DESKTOP, NULL, hInstance, NULL);

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

    while (1)
    {
		if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if(!GetMessage(&msg, NULL, 0, 0))
			return msg.wParam;
			TranslateMessage(&msg); 
			DispatchMessage(&msg);
		}
	    else
		{
			DrawScene(hdc);
			angle += 0.1;
			if (angle > 360.0) angle -= 360.0;
		}
    }

	return msg.wParam;
}
